/* AMX Mod X
*   Admin Help Plugin
*
* by the AMX Mod X Development Team
*  originally developed by tcquest78
*
* This file is part of AMX Mod X.
*
*
*  This program is free software; you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 2 of the License, or (at
*  your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software Foundation, 
*  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*  In addition, as a special exception, the author gives permission to
*  link the code of this program with the Half-Life Game Engine ("HL
*  Engine") and Modified Game Libraries ("MODs") developed by Valve, 
*  L.L.C ("Valve"). You must obey the GNU General Public License in all
*  respects for all of the code used other than the HL Engine and MODs
*  from Valve. If you modify this file, you may extend this exception
*  to your version of the file, but you are not obligated to do so. If
*  you do not wish to do so, delete this exception statement from your
*  version.
*/

#include <amxmodx>
#include <amxmisc>
#include <time>
#include <cstrike>
#include <celltrie>
#include <sqlx>

#define PLUGIN "Challenge"
#define VERSION "1.0"
#define AUTHOR "R3X"

#define DATE_FORMAT "%Y-%m-%d %H:%M:%S"

new Handle:gTuple;

new gszUsersTable[64];
new gszStatsTable[64];

new gszQuery[1024];
new const giMax = charsmax(gszQuery);

new gcvarSaveMethod;


new bool:gbLoaded[33];
new giPid[33];
new giPlayedTime[33];
new giFirstConnect[33];


new giWeaponStats[33][31][2];
new bool:gbWeaponChanged[33][31];
new bool:gbWeapListen[31] = {false, ...};

new Trie:gWeaponList;

getPlayedTime(id){
	return giPlayedTime[id] + get_user_time(id, 1);
}

addWeaponStat(id, wid, hs=0){
	giWeaponStats[id][wid][0]++;
	if(hs) giWeaponStats[id][wid][1]++;
		
	gbWeaponChanged[id][wid] = true;
}
stock SQL_PrepareString(const szQuery[], szOutPut[], size){
	copy(szOutPut, size, szQuery);
	replace_all(szOutPut, size, "'", "\'");
	replace_all(szOutPut,size, "`", "\`");	
}

public plugin_init() {
	register_plugin(PLUGIN, VERSION, AUTHOR);
	
	register_cvar("amx_challenge_host", "localhost");
	register_cvar("amx_challenge_user", "root");
	register_cvar("amx_challenge_pass", "root");
	register_cvar("amx_challenge_db", "challenge");
	register_cvar("amx_challenge_prefix", "");
	
	gcvarSaveMethod = register_cvar("amx_challenge_save", "1");
	
	register_logevent( "eventRoundEnd",2, "1=Round_End");
	register_event("DeathMsg", "eventDeathMsg", "a");
	
	register_dictionary("time.txt");
}
public plugin_cfg(){
	new szHost[64], szUser[64], szPass[64], szDB[64], szPrefix[32];
	get_cvar_string("amx_challenge_host", szHost, 63);
	get_cvar_string("amx_challenge_user", szUser, 63);
	get_cvar_string("amx_challenge_pass", szPass, 63);
	get_cvar_string("amx_challenge_db", szDB, 63);
	
	gTuple = SQL_MakeDbTuple(szHost, szUser, szPass, szDB, 1000);
	if(gTuple == Empty_Handle){
		set_fail_state("Brak polaczenia z baza danych!");
		return;
	}
	
	get_cvar_string("amx_challenge_prefix", szPrefix, 31);
	formatex(gszUsersTable, 63, "%susers", szPrefix);
	formatex(gszStatsTable, 63, "%sstats", szPrefix);
	
	tryCreateTables();
	
	setupWeaponList();
}

public plugin_end(){
	SQL_FreeHandle(gTuple);
	TrieDestroy(gWeaponList);
}

public plugin_natives(){
	register_library("Challenge");
	register_native("challenge_uses_stats", "_challenge_uses_stats");
	register_native("ch_get_user_stats", "_ch_get_user_stats");
	register_native("ch_get_user_played_time", "_ch_get_user_played_time");
	register_native("ch_get_user_first_connect", "_ch_get_user_first_connect");
}

tryCreateTables(){
	new iLen = 0;
	iLen += formatex(gszQuery[iLen], giMax-iLen, "CREATE TABLE IF NOT EXISTS `%s` (", gszUsersTable);
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"id	 INT(11) UNSIGNED AUTO_INCREMENT, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"steamid	VARCHAR(32) NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"nickname	VARCHAR(32) NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"ip		VARCHAR(32) NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"firstConnect	DATETIME NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"playedTime	INT(11)	NOT NULL DEFAULT 0, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"PRIMARY KEY(id) ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		")");
	
	SQL_ThreadQuery(gTuple, "handleCreateQuery", gszQuery);
	
	
	iLen = 0;
	iLen += formatex(gszQuery[iLen], giMax-iLen, "CREATE TABLE IF NOT EXISTS `%s` (", gszStatsTable);
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"pid	INT(11) UNSIGNED NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"wid	INT(11) NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"kills	INT(11) NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"hs	INT(11) NOT NULL, ");
	iLen += copy(gszQuery[iLen], giMax-iLen, 		"PRIMARY KEY(pid, wid), ");
	iLen += formatex(gszQuery[iLen], giMax-iLen, 		"FOREIGN KEY(pid) REFERENCES `%s`(id) ON DELETE CASCADE ",gszUsersTable);
	iLen += copy(gszQuery[iLen], giMax-iLen, 		")");
	
	SQL_ThreadQuery(gTuple, "handleCreateQuery", gszQuery);
}

setupWeaponList(){
	gWeaponList = TrieCreate();
	TrieSetCell(gWeaponList, "p228", CSW_P228);
	TrieSetCell(gWeaponList, "scout", CSW_SCOUT);
	TrieSetCell(gWeaponList, "grenade", CSW_HEGRENADE);
	TrieSetCell(gWeaponList, "xm1014", CSW_XM1014);
	TrieSetCell(gWeaponList, "mac10", CSW_MAC10);
	TrieSetCell(gWeaponList, "aug", CSW_AUG);
	TrieSetCell(gWeaponList, "elite", CSW_ELITE);
	TrieSetCell(gWeaponList, "fiveseven", CSW_FIVESEVEN);
	TrieSetCell(gWeaponList, "ump45", CSW_UMP45);
	TrieSetCell(gWeaponList, "sg550", CSW_SG550);
	TrieSetCell(gWeaponList, "galil", CSW_GALIL);
	TrieSetCell(gWeaponList, "famas", CSW_FAMAS);
	TrieSetCell(gWeaponList, "usp", CSW_USP);
	TrieSetCell(gWeaponList, "glock18", CSW_GLOCK18);
	TrieSetCell(gWeaponList, "awp", CSW_AWP);
	TrieSetCell(gWeaponList, "mp5navy", CSW_MP5NAVY);
	TrieSetCell(gWeaponList, "m249", CSW_M249);
	TrieSetCell(gWeaponList, "m3", CSW_M3);
	TrieSetCell(gWeaponList, "m4a1", CSW_M4A1);
	TrieSetCell(gWeaponList, "tmp", CSW_TMP);
	TrieSetCell(gWeaponList, "g3sg1", CSW_G3SG1);
	TrieSetCell(gWeaponList, "deagle", CSW_DEAGLE);
	TrieSetCell(gWeaponList, "sg552", CSW_SG552);
	TrieSetCell(gWeaponList, "ak47", CSW_AK47);
	TrieSetCell(gWeaponList, "knife", CSW_KNIFE);
	TrieSetCell(gWeaponList, "p90", CSW_P90);
}
//Create Handler
public handleCreateQuery(failstate, Handle:query, error[], errnum, data[], size){
	if(failstate != TQUERY_SUCCESS)
		log_amx("MySQL Creation error: %s",error);
}
//Insert user
public handleInsertQuery(failstate, Handle:query, error[], errnum, data[], size){
	if(failstate != TQUERY_SUCCESS){
		log_amx("MySQL Insert error: %s",error);
		return;
	}
	
	new id = data[0];
	if(!is_user_connected(id))
		return;
		
	giFirstConnect[id] = get_systime();
	giPlayedTime[id] = 0;
	client_authorized_sql(id, SQL_GetInsertId(query));
}
//Select pid
public handleSelectQuery(failstate, Handle:query, error[], errnum, data[], size){
	if(failstate != TQUERY_SUCCESS){
		log_amx("MySQL Select error: %s",error);
		return;
	}
	new id = data[0];
	if(!is_user_connected(id))
		return;
		
		
	if(SQL_NumRows(query)){
		new szDate[64];
		SQL_ReadResult(query, 1, szDate, 63);
		giFirstConnect[id] = parse_time(szDate, DATE_FORMAT);
		giPlayedTime[id] = SQL_ReadResult(query, 2);
		
		client_authorized_sql(id, SQL_ReadResult(query, 0));
	}else{
		new szSteamid[32], szName[64], szIp[32];
		get_user_authid(id, szSteamid, 31);
		get_user_name(id, szName, 31);
		get_user_ip(id, szIp, 31, 1);
		SQL_PrepareString(szName, szName, 63);
		
		formatex(gszQuery, giMax, "INSERT INTO `%s` VALUES (0, '%s', '%s', '%s', NOW(), 0)",
			gszUsersTable, szSteamid, szName, szIp);
		SQL_ThreadQuery(gTuple, "handleInsertQuery", gszQuery, data, size);
	}
}
//Select pid
public handleLoadStatsQuery(failstate, Handle:query, error[], errnum, data[], size){
	if(failstate != TQUERY_SUCCESS){
		log_amx("MySQL Load Stats error: %s",error);
		return;
	}
	
	new id = data[0];
	if(!is_user_connected(id))
		return;
		
	new wid;
	while(SQL_MoreResults(query)){
		wid = SQL_ReadResult(query, 0);
		giWeaponStats[id][wid][0] = SQL_ReadResult(query, 1);
		giWeaponStats[id][wid][1] = SQL_ReadResult(query, 2);
		
		SQL_NextRow(query);
	}
	
	gbLoaded[id] = true;
}
//Save query
public handleSaveQuery(failstate, Handle:query, error[], errnum, data[], size){
	if(failstate != TQUERY_SUCCESS){
		log_amx("MySQL Save error: %s",error);
		return;
	}
}
	
trySelectPid(id, const szField[], const szAuth[]){
	formatex(gszQuery, giMax, "SELECT id, firstConnect, playedTime  FROM `%s` WHERE %s = '%s'",
			gszUsersTable, szField, szAuth);
	
	new data[1];
	data[0] = id;
	SQL_ThreadQuery(gTuple, "handleSelectQuery", gszQuery, data, sizeof(data));
}

public client_connect(id){
	gbLoaded[id] = false;
	giPid[id] = 0;
}
public client_disconnect(id){
	if(gbLoaded[id]){
		saveData(id);
	}
}
public client_putinserver(id){
	clearWeaponStats(id);
	
	new szAuth[64], szField[32];
	switch(get_pcvar_num(gcvarSaveMethod)){
		case 3:{
			get_user_ip(id, szAuth, 31, 1);
			copy(szField, 31, "ip");
		}
		case 2:{
			get_user_name(id, szAuth, 31);
			SQL_PrepareString(szAuth, szAuth, 63);
			copy(szField, 31, "nickname");
		}
		default:{
			get_user_authid(id, szAuth, 31);
			copy(szField, 31, "steamid");
		}
	}
	trySelectPid(id, szField, szAuth);
}

public client_authorized_sql(id, pid){
	giPid[id] = pid;
	loadWeaponStats(id);
}
clearWeaponStats(id){
	for(new i=0;i<31;i++){
		giWeaponStats[id][i][0] = 0;
		giWeaponStats[id][i][1] = 0;
	}
}
loadWeaponStats(id){
	if(!is_user_connected(id) || !giPid[id]) return;
	
	formatex(gszQuery, giMax, "SELECT wid,kills,hs FROM `%s` WHERE pid = %d",
			gszStatsTable, giPid[id]);
	
	new data[1];
	data[0] = id;
	SQL_ThreadQuery(gTuple, "handleLoadStatsQuery", gszQuery, data, sizeof(data));
}

saveData(id){
	if(gbLoaded[id]){
		savePlayedTime(id);
		saveWeaponData(id);
	}
}
savePlayedTime(id){
	formatex(gszQuery, giMax, "UPDATE `%s` SET playedTime = %d WHERE id = %d;",
			gszUsersTable, getPlayedTime(id), giPid[id]);
	SQL_ThreadQuery(gTuple, "handleSaveQuery", gszQuery);
}
saveWeaponData(id){
	new iLen=0;
	for(new i=1;i<31;i++)
		if(gbWeaponChanged[id][i]){
			iLen += formatex(gszQuery[iLen], giMax-iLen, 
			"INSERT INTO `%s` VALUES (%d, %d, %d, %d) ON DUPLICATE KEY UPDATE kills=%d,hs=%d;",
				gszStatsTable, giPid[id], i, giWeaponStats[id][i][0], giWeaponStats[id][i][1],
				giWeaponStats[id][i][0], giWeaponStats[id][i][1]);
		}
	SQL_ThreadQuery(gTuple, "handleSaveQuery", gszQuery);
}
public eventRoundEnd(){
	for(new id=1;id<33;id++){
		if(is_user_connected(id))
			saveData(id);
			
		for(new j=1;j<31;j++)
			gbWeaponChanged[id][j] = false;
	}
}
public eventDeathMsg(){
	new killer = read_data(1);
	if(!is_user_connected(killer))
		return PLUGIN_CONTINUE;
		
	new szWeapon[32];
	read_data(4, szWeapon, 31);
	
	new wid;
	if(!TrieGetCell(gWeaponList, szWeapon, wid) || !gbWeapListen[wid])
		return PLUGIN_CONTINUE;
	
	new victim = read_data(2);
	if(cs_get_user_team(victim) == cs_get_user_team(killer))
		return PLUGIN_CONTINUE;
		
	new hs = read_data(3);
	addWeaponStat(killer, wid, hs);
	return PLUGIN_CONTINUE;
}



//Natives
validWeapon(wid){
	return (wid > 0 && wid <= CSW_P90);
}
public _challenge_uses_stats(plugin, params){
	if(params < 1)
		return 0;
	
	new wid = get_param(1);
	if(validWeapon(wid))
	{
		gbWeapListen[wid] = true;
		return 1;
	}
	return 0;
}

public _ch_get_user_stats(plugin, params){
	if(params < 3)
		return 0;
	
	new id = get_param(1);
	if(!is_user_connected(id))
		return 0;
	
	new wid = get_param(2);
	if(!validWeapon(wid))
		return 0;
		
	new hs = !!get_param(3);
	return giWeaponStats[id][wid][hs];
}

public _ch_get_user_played_time(plugin, params){
	if(params < 1)
		return 0;
		
	new id = get_param(1);
	if(!is_user_connected(id))
		return 0;
	return getPlayedTime(id);
}

public _ch_get_user_first_connect(plugin, params){
	if(params < 1)
		return 0;
		
	new id = get_param(1);
	if(!is_user_connected(id))
		return 0;
		
	return giFirstConnect[id];
}
